using System;
using System.Web;
using System.Net;
using System.Data;
using System.Web.UI;
using System.Web.Security;
using System.Net.Security;
using System.Configuration;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls.WebParts;
using System.Security.Cryptography.X509Certificates;

using gov.nih.nlm.kswebp2;
using gov.nih.nlm.kscas;

// Sample code to connect to UMLS and leverage one of their services.
// In this case, obtaining spelling suggestions.
//
// Author: Frank DeFalco 
// IT Manager, Market Intelligence and Informatics
// Ortho-Clinical Diagnostics
// Johnson & Johnson
public partial class _Default : System.Web.UI.Page
{
	protected void Page_Load(object sender, EventArgs e)
	{
	}

	protected void ButtonExecute_Click(object sender, EventArgs e)
	{
		string search_string = TextCriteria.Text;

		// the following configuration settings come from the web.config file
		// the values will need to be updated in the web.config file in order for the sample to operate properly
		string username = ConfigurationManager.AppSettings["security.username"];
		string password = ConfigurationManager.AppSettings["security.password"];
		string my_certificate_location = ConfigurationManager.AppSettings["security.certificate.location"];
		string proxy_ticket_url = ConfigurationManager.AppSettings["security.proxyticket.url"];

		AuthorizationService authservice = new AuthorizationService();

		// authorize this request with my certificate
		string my_certificate_path = Request.MapPath(my_certificate_location);
		X509Certificate my_certificate = new X509Certificate(my_certificate_path, password);
		authservice.ClientCertificates.Add(my_certificate);

		// Because the ssl certificate used by nlm is not from a trusted certificate authority we have to enhance 
		// the validation routine to support our needs.
		// 
		// This implementation can introduce security concerns and should be evaluated by any organzation leveraging 
		// this sample code.
		//
		ServicePointManager.ServerCertificateValidationCallback +=
			delegate(object originator, X509Certificate certificate, X509Chain chain, SslPolicyErrors sslPolicyErrors)
			{
				bool validationResult = true;

				// << add validation routine here >>

				return validationResult;
			};

		// Obtain the proxy granting ticket.
		// This ticket is then used to obtain the proxy ticket for each individual request.
		string proxy_granting_ticket = authservice.getProxyGrantTicket(username, password);

		string proxy_ticket;

		UMLSKSService service = new UMLSKSService();

		// Obtain the value representing the current UMLS release.
		CurrentUMLSRequest umls_release_request = new CurrentUMLSRequest();
		proxy_ticket = authservice.getProxyTicket(proxy_granting_ticket, proxy_ticket_url);
		umls_release_request.casTicket = proxy_ticket;
		string umls_release = service.getCurrentUMLSVersion(umls_release_request);

		// Obtain spelling suggestions for a term.
		SuggestSpellingRequest suggest_spelling_request = new SuggestSpellingRequest();
		proxy_ticket = authservice.getProxyTicket(proxy_granting_ticket, proxy_ticket_url);
		suggest_spelling_request.casTicket = proxy_ticket;
		suggest_spelling_request.term = search_string;
		suggest_spelling_request.release = umls_release;
		SpellingSuggestionsGroup ssg = service.suggestSpelling(suggest_spelling_request);

		// collect the results 
		System.Collections.Generic.List<string>  results = new System.Collections.Generic.List<string>();
		foreach (string s in ssg.contents)
		{
			if (!results.Contains(s.ToLower()))
			{
				results.Add(s.ToLower());
			}
		}

		// bind the results to the dataList object
		this.DataListSuggestions.DataSource = results;
		this.DataListSuggestions.DataBind();
	}
}
